﻿#include <iostream>
#include <algorithm>
#include "MeshLoader.h"


const std::vector<Node>& MeshLoader::getNodes() {
    return Nodes;
}

const std::vector<FiniteElement>& MeshLoader::getFiniteElement() {
    return element;
}


std::vector<int> MeshLoader::getFiniteElementID(int _id1, int _id2, int _id3) {
    std::vector<int> res;
    auto po = [&_id1, &_id2, &_id3, &res](const FiniteElement& elem) {
        if ((std::find(elem.NodeID.begin(), elem.NodeID.end(), _id1) != elem.NodeID.end()) &&
            (std::find(elem.NodeID.begin(), elem.NodeID.end(), _id2) != elem.NodeID.end()) &&
            (std::find(elem.NodeID.begin(), elem.NodeID.end(), _id3) != elem.NodeID.end()) && 
            elem.NodeID.size() == 3) {
            res.push_back(elem.FiniteElementID);
        }
    };

    std::for_each(element.begin(), element.end(),po);
    return res;
}

std::vector<int> MeshLoader::getFiniteElementEdge(int _id1, int _id2) {
    std::vector<int> res;
    auto po = [&_id1, &_id2, &res](const FiniteElement& elem) {
        if ((std::find(elem.NodeID.begin(), elem.NodeID.end(), _id1) != elem.NodeID.end()) &&
            (std::find(elem.NodeID.begin(), elem.NodeID.end(), _id2) != elem.NodeID.end()) &&
            elem.NodeID.size() == 3) {
            res.push_back(elem.FiniteElementID);
        }
    };

    std::for_each(element.begin(), element.end(),po);
    return res;
}

std::vector<int> MeshLoader::getNodesId(int _id) {
    std::vector<int> res;
    auto cur = element.begin();

    auto po = [_id](const FiniteElement& elem) {
        return elem.GeometryAreaID == _id && elem.NodeID.size() == 3;
    };

    while (cur != element.end()) {
        cur = std::find_if(cur, element.end(), po);
        if (cur != element.end()) {
            for (auto node : cur->NodeID) {
                Node nodeId = Nodes[node];
                res.push_back(nodeId.NodeID);
            }
            ++cur;
        }
    }
    return std::vector<int>(res.begin(), res.end());
}

std::vector<int> MeshLoader::getFiniteElementIdSurface(int _id) {
    std::vector<int> res;
    auto po = [&_id, &res](const FiniteElement& elem) {
        if (elem.GeometryAreaID == _id && elem.NodeID.size() == 3) {
            res.push_back(elem.FiniteElementID);
        }
    };

    std::for_each(element.begin(), element.end(), po);
    return res;
}

std::vector<int> MeshLoader::getFiniteElementIDVol(int _id) {
    std::vector<int> res;
    auto po = [&_id, &res](const FiniteElement& elem) {
        if (elem.GeometryAreaID == _id && elem.NodeID.size() == 4) {
            res.push_back(elem.FiniteElementID);
        }
    };

    std::for_each(element.begin(), element.end(), po);
    return res;
}

void MeshLoader::printNode() {
    std::cout << Nodes;
}

void MeshLoader::printFiniteElement() {
    std::cout << element;
}







